@@ -1,6 +1,6 @@
 /* configure.c -- Configuration files management functions.  */
 
-/* Copyright (C) 1993, 1994, 1995, 1996 Free Software Foundation, Inc.
+/* Copyright (C) 1993-1998 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Written by Tudor Hulubei and Andrei Pitis.  */
-
+/* $Id: configure.c,v 1.4 1998/03/10 05:12:11 tudor Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -30,7 +30,7 @@
 #endif /* !HAVE_STDLIB_H */
 
 #include <stdio.h>
-
+#include <ctype.h>
 #include <sys/types.h>
 #include "file.h"
 #include <ctype.h>
@@ -147,7 +147,7 @@ configuration_getvarinfo(var_name, dest, fields, seek)
 	if (seek == NO_SEEK)
 	    buf[32] = 0;        /* Just in case... */
 
-	if (!is_print(*buf))
+	if (!isprint(*buf))
 	    return;
 
 	if (seek == NO_SEEK || strcmp(var_name, buf) == 0)
