@@ -1,6 +1,6 @@
 /* gitps.c -- A process viewer/killer utility.  */
 
-/* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
+/* Copyright (C) 1993-1998 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Written by Tudor Hulubei and Andrei Pitis.  */
-
+/* $Id: gitps.c,v 1.8 1998/03/13 16:04:00 tudor Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -59,21 +59,22 @@
 extern int errno;
 #endif /* !errno */
 
+#include <assert.h>
+
 #include "stdc.h"
 #include "xstring.h"
 #include "xmalloc.h"
+#include "getopt.h"
 #include "tty.h"
 #include "window.h"
 #include "configure.h"
 #include "tilde.h"
+#include "signals.h"
 #include "misc.h"
 
 
-int SCREEN_X;
-int SCREEN_Y;
-
 #define MAX_KEYS        2048
-#define MAX_PROCESSES   1024
+#define MAX_LINE	2048
 #define PS_FIELDS         12
 
 
@@ -130,72 +131,153 @@ int AnsiColors = OFF;
 #endif  /* !HAVE_LINUX */
 
 
-char cSection[]  = "[GITPS-Color]";
-char bwSection[] = "[GITPS-Monochrome]";
+char color_section[]  = "[GITPS-Color]";
+char monochrome_section[] = "[GITPS-Monochrome]";
 int  processes;
 int  PID_index;
-int  signal_type = 11;  /* index of SIGTERM in signals table */
 
-pid_t pid;
 char *home;
 char *program;
-char *header_text;
-char *TempDirectory;
+char *ps_cmd;
+char *temporary_directory;
+char header_text[MAX_LINE];
 int UseLastScreenChar;
 int StartupScrollStep;
 char *stdout_log_name;
 char *stderr_log_name;
-char *ps_vect[MAX_PROCESSES];
+char **ps_vect;
 char *screen;
 char *global_buf;
 int first_on_screen, current_process, scroll_step;
-window_t *title_win, *header_win, *screen_win, *status_win;
-#ifdef HAVE_GCC
-static char title_text[] = " "PRODUCT" "VERSION" - Process Viewer/Killer";
-#else
-static char title_text[] = " GNU Interactive Tools 4.3.16 - Process Viewer/Killer";
-#endif /* !HAVE_GCC */
-static char *GitPsModeHelp;
+window_t *title_window, *header_window, *processes_window, *status_window;
+static char *title_text;
+static char *help;
 static char no_perm[] = "not owner !";
 static char no_proc[] = "no such process ! (REFRESH recommended)";
 
 
-struct SIGNAL
+typedef struct
 {
-    char signame[8];
+    char signame[10];
     int  signal;
-};
+} xsignal_t;
 
-static struct SIGNAL signals[] =
+int signal_type = 0;
+
+static xsignal_t sigdesc[] =
 {
-    { "SIGHUP ", SIGHUP  },
-    { "SIGINT ", SIGINT  },
-    { "SIGQUIT", SIGQUIT },
-    { "SIGILL ", SIGILL  },
-    { "SIGFPE ", SIGFPE  },
-    { "SIGKILL", SIGKILL },
-    { "SIGUSR1", SIGUSR1 },
-    { "SIGSEGV", SIGSEGV },
-    { "SIGUSR2", SIGUSR2 },
-    { "SIGPIPE", SIGPIPE },
-    { "SIGALRM", SIGALRM },
-    { "SIGTERM", SIGTERM },
-    { "SIGCHLD", SIGCHLD },
+    { "SIGTERM  ", SIGTERM   },
+    { "SIGABRT  ", SIGABRT   },
+    { "SIGALRM  ", SIGALRM   },
 
-#ifdef SIGSTOP
-    { "SIGSTOP", SIGSTOP },
+#ifdef SIGBUS
+    { "SIGBUS   ", SIGBUS    },
 #endif
 
-#ifdef SIGTSTP
-    { "SIGTSTP", SIGTSTP },
+#ifdef SIGCHLD
+    { "SIGCHLD  ", SIGCHLD   },
+#endif
+
+#ifdef SIGCLD
+    { "SIGCLD   ", SIGCLD    },
 #endif
 
 #ifdef SIGCONT
-    { "SIGCONT", SIGCONT },
+    { "SIGCONT  ", SIGCONT   },
 #endif
 
-    { "SIGABRT", SIGABRT },
-    { "SIGTRAP", SIGTRAP },
+#ifdef SIGEMT
+    { "SIGEMT   ", SIGEMT    },
+#endif
+
+#ifdef SIGFPE
+    { "SIGFPE   ", SIGFPE    },
+#endif
+
+    { "SIGHUP   ", SIGHUP    },
+
+#ifdef SIGILL
+    { "SIGILL   ", SIGILL    },
+#endif
+
+#ifdef SIGINFO
+    { "SIGINFO  ", SIGINFO   },
+#endif
+
+    { "SIGINT   ", SIGINT    },
+
+#ifdef SIGIO
+    { "SIGIO    ", SIGIO     },
+#endif
+
+#ifdef SIGIOT
+    { "SIGIOT   ", SIGIOT    },
+#endif
+
+    { "SIGKILL  ", SIGKILL   },
+    { "SIGPIPE  ", SIGPIPE   },
+
+#ifdef SIGPOLL
+    { "SIGPOLL  ", SIGPOLL   },
+#endif
+
+#ifdef SIGPROF
+    { "SIGPROF  ", SIGPROF   },
+#endif
+
+#ifdef SIGPWR
+    { "SIGPWR   ", SIGPWR    },
+#endif
+
+    { "SIGQUIT  ", SIGQUIT   },
+    { "SIGSEGV  ", SIGSEGV   },
+
+#ifdef SIGSTOP
+    { "SIGSTOP  ", SIGSTOP   },
+#endif
+
+#ifdef SIGSYS
+    { "SIGSYS   ", SIGSYS    },
+#endif
+
+#ifdef SIGTRAP
+    { "SIGTRAP  ", SIGTRAP   },
+#endif
+
+#ifdef SIGTSTP
+    { "SIGTSTP  ", SIGTSTP   },
+#endif
+
+#ifdef SIGTTIN
+    { "SIGTTIN  ", SIGTTIN   },
+#endif
+
+#ifdef SIGTTOU
+    { "SIGTTOU  ", SIGTTOU   },
+#endif
+
+#ifdef SIGURG
+    { "SIGURG   ", SIGURG    },
+#endif
+
+    { "SIGUSR1  ", SIGUSR1   },
+    { "SIGUSR2  ", SIGUSR2   },
+
+#ifdef SIGVTALRM
+    { "SIGVTALRM", SIGVTALRM },
+#endif
+
+#ifdef SIGWINCH
+    { "SIGWINCH ", SIGWINCH  },
+#endif
+
+#ifdef SIGXCPU
+    { "SIGXCPU  ", SIGXCPU   },
+#endif
+
+#ifdef SIGXFSZ
+    { "SIGXFSZ  ", SIGXFSZ   },
+#endif
 };
 
 
@@ -264,65 +346,84 @@ char built_in[BUILTIN_OPERATIONS][MAX_BUILTIN_NAME] =
 void
 remove_log()
 {
-    if (stdout_log_name) unlink(stdout_log_name);
-    if (stderr_log_name) unlink(stderr_log_name);
+    if (stdout_log_name)
+	unlink(stdout_log_name);
+
+    if (stderr_log_name)
+	unlink(stderr_log_name);
 }
 
 
 void
-settitle()
+set_title()
 {
-    memset(global_buf, ' ', SCREEN_X);
-    memcpy(global_buf, title_text, strlen(title_text));
+    memset(global_buf, ' ', tty_columns);
+    memcpy(global_buf, title_text,
+	   min(tty_columns, (int)strlen(title_text)));
 
     tty_colors(TitleBrightness, TitleForeground, TitleBackground);
 
-    window_goto(title_win, 0, 0);
-    window_puts(global_buf, SCREEN_X);
+    window_goto(title_window, 0, 0);
+    window_puts(title_window, global_buf, tty_columns);
 }
 
 
 void
-setheader()
+set_header()
 {
-    memset(global_buf, ' ', SCREEN_X);
-    memcpy(global_buf, header_text, strlen(header_text));
+    memset(global_buf, ' ', tty_columns);
+    memcpy(global_buf, header_text,
+	   min(tty_columns, (int)strlen(header_text)));
 
     tty_colors(HeaderBrightness, HeaderForeground, HeaderBackground);
 
-    window_goto(header_win, 0, 0);
-    window_puts(global_buf, SCREEN_X);
+    window_goto(header_window, 0, 0);
+    window_puts(header_window, global_buf, tty_columns);
 }
 
 
 void
-setstatus(what)
+set_status(what)
    char *what;
 {
-    memset(global_buf, ' ', SCREEN_X);
+    memset(global_buf, ' ', tty_columns);
 
     if (what)
-	memcpy(global_buf, what, strlen(what));
+	memcpy(global_buf, what, min(tty_columns, (int)strlen(what)));
     else
-	memcpy(global_buf, GitPsModeHelp, strlen(GitPsModeHelp));
+	memcpy(global_buf, help, min(tty_columns, (int)strlen(help)));
 
     tty_colors(StatusBrightness, StatusForeground, StatusBackground);
 
-    window_goto(status_win, 0, 0);
-    window_puts(global_buf, SCREEN_X - (sizeof(signals[0].signame) - 1) - 1);
+    window_goto(status_window, 0, 0);
+
+    if (tty_columns < (int)((sizeof(sigdesc[0].signame) - 1) + 1))
+	window_puts(status_window, global_buf, tty_columns);
+    else
+    {
+	global_buf[tty_columns - 1 - (sizeof(sigdesc[0].signame)-1) - 1] = ' ';
+	window_puts(status_window, global_buf,
+		    tty_columns - (sizeof(sigdesc[0].signame) - 1) - 1);
+    }
 }
 
 
 void
-setsignal()
+set_signal()
 {
-    int len = sizeof(signals[0].signame) - 1;
+    int len = sizeof(sigdesc[0].signame) - 1;
 
-    tty_colors(StatusBrightness, StatusForeground, StatusBackground);
+    if (tty_columns > len)
+    {
+	tty_colors(StatusBrightness, WHITE, StatusBackground);
+	window_goto(status_window, 0, tty_columns - len - 1);
+	window_puts(status_window, sigdesc[signal_type].signame, len);
 
-    window_goto(status_win, 0, SCREEN_X - len - 1);
-    window_puts(signals[signal_type].signame, len);
-    window_putc(' ');
+	if (UseLastScreenChar == ON)
+	    window_putc(status_window, ' ');
+    }
+
+    window_goto(status_window, 0, tty_columns - 1);
 }
 
 
@@ -331,12 +432,11 @@ free_ps_list()
 {
     int i;
 
-    for (i = 0; i < MAX_PROCESSES; i++)
-	if (ps_vect[i])
-	{
-	    xfree(ps_vect[i]);
-	    ps_vect[i] = NULL;
-	}
+    for (i = 0; i < processes; i++)
+    {
+	xfree(ps_vect[i]);
+	ps_vect[i] = NULL;
+    }
 }
 
 
@@ -349,12 +449,16 @@ read_ps_line(ps_output, line)
     char *ok;
     size_t lastchar;
 
-    ok = fgets(line, SCREEN_X + 1, ps_output);
+    ok = fgets(line, MAX_LINE - 1, ps_output);
+
+    if (ok == NULL)
+	return NULL;
 
     if (line[lastchar = strlen(line) - 1] == '\n')
 	line[lastchar] = 0;
     else
-	while ((c = fgetc(ps_output)) != '\n' && c != EOF);
+	while ((c = fgetc(ps_output)) != '\n' && c != EOF)
+	    ;
 
     return ok;
 }
@@ -392,24 +496,36 @@ kill_process(process_index)
     int process_index;
 {
     int i;
-    char pidstr[32];
-    char *p = ps_vect[process_index];
+    char *p;
+    char pidstr[128];
 
-    if (p == NULL)
-	return 0;
+    assert(process_index < processes);
+    p = ps_vect[process_index];
+    assert(p);
 
     for (i = 0; i < PID_index; i++)
     {
-	while (isspace(*p)) p++;
-	if (memcmp(p, "PID", 3) == 0) return i;
-	while (!isspace(*p)) p++;
+	while (isspace(*p))
+	    p++;
+
+	if (memcmp(p, "PID", 3) == 0)
+	    return i;
+
+	while (!isspace(*p))
+	    p++;
     }
 
     i = 0;
-    while (isspace(*p)) p++;
-    while (!isspace(*p)) pidstr[i++] = *p++;
+
+    while (isspace(*p))
+	p++;
+
+    while (!isspace(*p))
+	pidstr[i++] = *p++;
+
     pidstr[i] = 0;
-    return !kill(atoi(pidstr), signals[signal_type].signal);
+
+    return !kill(atoi(pidstr), sigdesc[signal_type].signal);
 }
 
 
@@ -417,14 +533,15 @@ void
 build_ps_list(ps_output)
     FILE *ps_output;
 {
-    int i = 0;
+    int i;
+    char line[MAX_LINE];
 
-    do
-	ps_vect[i] = xmalloc(SCREEN_X + 1);
-    while (read_ps_line(ps_output, ps_vect[i++]));
+    for (i = 0; read_ps_line(ps_output, line); i++)
+    {
+	ps_vect = (char **)xrealloc(ps_vect, (i + 1) * sizeof(char *));
+	ps_vect[i] = xstrdup(line);
+    }
 
-    xfree(ps_vect[--i]);
-    ps_vect[i] = NULL;
     processes = i;
 }
 
@@ -433,8 +550,11 @@ void
 update_process(process, update_color)
     int process, update_color;
 {
-    memset(global_buf, ' ', SCREEN_X);
-    memcpy(global_buf, ps_vect[process], strlen(ps_vect[process]));
+    assert(process < processes);
+
+    memset(global_buf, ' ', tty_columns);
+    memcpy(global_buf, ps_vect[process],
+	   min(tty_columns, (int)strlen(ps_vect[process]) - 1));
 
     if (update_color)
     {
@@ -452,8 +572,9 @@ update_process(process, update_color)
 	}
     }
 
-    window_goto(screen_win, process - first_on_screen, 0);
-    window_puts(global_buf, SCREEN_X);
+    window_goto(processes_window, process - first_on_screen, 0);
+    window_puts(processes_window, global_buf, tty_columns);
+    window_goto(status_window, 0, tty_columns - 1);
 }
 
 
@@ -464,35 +585,38 @@ update_all()
 
     tty_colors(ScreenBrightness, ScreenForeground, ScreenBackground);
 
-    window_goto(screen_win, 0, 0);
+    if (tty_lines <= 4)
+	return;
+
+    window_goto(processes_window, 0, 0);
 
     for (i = first_on_screen;
-	 i < processes && (i - first_on_screen < SCREEN_Y - 3); i++)
+	 i < processes && (i - first_on_screen < tty_lines - 3); i++)
 	    if (i != current_process)
 		update_process(i, OFF);
 	    else
-		window_goto(screen_win, i - first_on_screen, 0);
+		window_goto(processes_window, i - first_on_screen, 0);
 
     update_process(current_process, ON);
 
     tty_colors(ScreenBrightness, ScreenForeground, ScreenBackground);
 
-    memset(global_buf, ' ', SCREEN_X);
+    memset(global_buf, ' ', tty_columns);
 
-    for (; i - first_on_screen < SCREEN_Y - 3; i++)
+    for (; i - first_on_screen < tty_lines - 3; i++)
     {
-	window_goto(screen_win, i - first_on_screen, 0);
-	window_puts(global_buf, SCREEN_X);
+	window_goto(processes_window, i - first_on_screen, 0);
+	window_puts(processes_window, global_buf, tty_columns);
     }
 
-    window_goto(screen_win, current_process-first_on_screen, SCREEN_X-1);
+    window_goto(status_window, 0, tty_columns - 1);
 }
 
 
 void
 clean_up()
 {
-    tty_exit(NULL);
+    tty_end(NULL);
     remove_log();
 }
 
@@ -509,13 +633,11 @@ fatal(postmsg)
 
 int
 ps(args)
-    char *args;
+    char **args;
 {
-    char *ps_cmd;
     FILE *stdout_log, *stderr_log;
 
     /* See the comment in system.c on closing tty descriptors.  */
-
     int old_stdout = dup(1);
     int old_stderr = dup(2);
 
@@ -525,12 +647,28 @@ ps(args)
     stdout_log = fopen(stdout_log_name, "w");
     stderr_log = fopen(stderr_log_name, "w");
 
-    ps_cmd = xmalloc(16 + (args ? strlen(args) : 0) + 1);
+    if (ps_cmd)
+	xfree(ps_cmd);
 
     if (args)
-	sprintf(ps_cmd, "ps %s", args);
+    {
+	int i, bytes = 0;
+
+	for (i = 0; args[i]; i++)
+	    bytes += 1 + strlen(args[i]);
+
+	ps_cmd = xmalloc(16 + bytes + 1);
+
+	strcpy(ps_cmd, "ps");
+
+	for (i = 0; args[i]; i++)
+	{
+	    strcat(ps_cmd, " ");
+	    strcat(ps_cmd, args[i]);
+	}
+    }
     else
-	sprintf(ps_cmd, "ps");
+	ps_cmd = xstrdup("ps");
 
     if (system(ps_cmd) != 0)
     {
@@ -543,12 +681,13 @@ ps(args)
 	close(old_stdout);
 	close(old_stderr);
 
-	fprintf(stderr, "%s: invalid command line.\n", program);
+	fprintf(stderr, "%s: invalid command line for ps(1).\n", program);
+	fprintf(stderr, "%s: the command was: `%s'.\n", program, ps_cmd);
+	fprintf(stderr, "%s: see the ps(1) man page for details.\n", program);
+
 	return 0;
     }
 
-    xfree(ps_cmd);
-
     fclose(stdout_log);
     fclose(stderr_log);
 
@@ -562,14 +701,6 @@ ps(args)
 }
 
 
-RETSIGTYPE
-panic(signum)
-    int signum;
-{
-    fatal_signal(signum);
-}
-
-
 int
 read_keys(keys)
     int keys;
@@ -635,55 +766,238 @@ read_keys(keys)
 }
 
 
+void
+resize(resize_required)
+    int resize_required;
+{
+    int display_title = OFF;
+    int display_header = OFF;
+    int display_processes = OFF;
+    int old_tty_lines = tty_lines;
+    int old_tty_columns = tty_columns;
+
+    tty_resize();
+
+    /* Don't resize, unless absolutely necessary.  */
+    if (!resize_required)
+	if (tty_lines == old_tty_lines && tty_columns == old_tty_columns)
+	    return;
+
+#ifdef HAVE_LINUX
+    if (LinuxConsole)
+	screen = xrealloc(screen, 4 + tty_columns * tty_lines * 2);
+#endif  /* HAVE_LINUX */
+
+    /* Watch out for special cases (tty_lines < 7) because some
+     * components will no longer fit.
+     *
+     * The title needs one line.  The panels need four.  There is one
+     * more line for the input line and one for the status.
+     *
+     * Cases (lines available):
+     *      1 line:	the status
+     *      2 lines:	title + status
+     *    3-4 lines:	title + header + status
+     *	 >= 5 lines:	everything
+     */
+
+    global_buf = xrealloc(global_buf, tty_columns + 1);
+
+    current_process = min(current_process, processes - 1);
+    first_on_screen = max(0, current_process - (tty_lines - 3) + 1);
+    assert(first_on_screen >= 0);
+
+    window_resize(status_window, 0, tty_lines - 1, 1, tty_columns);
+
+    if (tty_lines >= 2)
+	display_title = ON;
+
+    if (tty_lines >= 3)
+	display_header = ON;
+
+    if (tty_lines >= 5)
+    {
+	display_processes = ON;
+
+	if (StartupScrollStep <= 0 || StartupScrollStep >= (tty_lines - 3) - 1)
+	    scroll_step = (tty_lines - 3) / 2;
+	else
+	    scroll_step = StartupScrollStep;
+    }
+
+    window_resize(title_window, 0, 0, display_title ? 1 : 0, tty_columns);
+    window_resize(header_window, 0, 1, display_header ? 1 : 0, tty_columns);
+    window_resize(processes_window, 0, 2,
+		  display_processes ? (tty_lines - 3) : 0, tty_columns);
+}
+
+
+/*
+ * Resize (if necessary) and then refresh all gitps' components.
+ */
+
+void
+refresh(signum)
+    int signum;
+{
+    resize(0);
+
+    if (signum == SIGCONT)
+    {
+	/* We were suspended.  Switch back to noncanonical mode.  */
+	tty_set_mode(TTY_NONCANONIC);
+	tty_defaults();
+    }
+
+    if (tty_lines == 4)
+    {
+	/* We know that when this is the case, the processes will not
+	   be displayed and there will be an empty line that
+	   potentially can contain some junk.  */
+	tty_defaults();
+	tty_clear();
+    }
+
+    set_title();
+    set_header();
+    set_status((char *)NULL);
+    set_signal();
+    update_all();
+    tty_update();
+
+    if (signum == SIGCONT)
+	tty_update_title(ps_cmd);
+}
+
+
+void
+hide()
+{
+    tty_set_mode(TTY_CANONIC);
+    tty_defaults();
+    tty_put_screen(screen);
+}
+
+
+void
+clock_refresh()
+{
+}
+
+
+void
+usage()
+{
+    printf("usage: %s [-hvilcbp]\n", program);
+    printf(" -h         print this help message\n");
+    printf(" -v         print the version number\n");
+    printf(" -i         print the installation directory\n");
+    printf(" -c         use ANSI colors\n");
+    printf(" -b         don't use ANSI colors\n");
+    printf(" -l         don't use the last screen character\n");
+    printf(" -p         pass the remaining arguments to ps(1)\n");
+}
+
+
 int
 main(argc, argv)
     int argc;
     char *argv[];
 {
+    char *tmp;
     int key, keys;
     tty_key_t *ks;
     FILE *stdout_log;
     int repeat_count;
-    char *tmp, *data = NULL;
+    char **arguments;
+    int i, no_of_arguments, exit_code = 0, r1, r2;
     int need_update, need_update_all, old_current_process;
+    int c, ansi_colors = -1, use_last_screen_character = ON;
 
 
+    /* Make sure we don't get signals before we are ready to handle
+       them.  */
+    signals_init();
+
     program = argv[0];
-    pid     = getpid();
 
     home = getenv("HOME");
     if (home == NULL)
 	home = ".";
 
-    get_tty_name();
+    compute_directories();
     get_login_name();
 
-    tty_get_capabilities();
-    tty_kbdinit(TTY_FULL_INPUT);
+    if (getenv("COLORTERM") != NULL)
+	ansi_colors = ON;
 
-    signal(SIGTERM, panic);
-    signal(SIGINT , panic);
-    signal(SIGQUIT, panic);
-    signal(SIGSEGV, panic);
-    signal(SIGHUP,  panic);
+    /* Parse the command line.  */
+    while ((c = getopt(argc, argv, "hvicblp")) != -1)
+	switch (c)
+	{
+	    case 'h':
+		/* Help request.  */
+		usage();
+		return 0;
 
-    signal(SIGILL,  SIG_IGN);
-    signal(SIGTRAP, SIG_IGN);
-    signal(SIGABRT, SIG_IGN);
-    signal(SIGUSR1, SIG_IGN);
-    signal(SIGUSR2, SIG_IGN);
+	    case 'v':
+		/* Version number request.  */
+		printf("%s %s\n", PRODUCT, VERSION);
+		return 0;
 
-#ifdef SIGTSTP
-    signal(SIGTSTP, SIG_IGN);
-#endif
+	    case 'i':
+		/* Installation directory request.  */
+		printf("%s is installed in %s.\n", PRODUCT, PREFIX);
+		return 0;
 
-#ifdef SIGCONT
-    signal(SIGCONT, SIG_IGN);
-#endif
+	    case 'c':
+		/* Force git to use ANSI color sequences.  */
+		ansi_colors = ON;
+		break;
 
-    signal(SIGALRM, SIG_IGN);
-    signal(SIGPIPE, SIG_IGN);
-    signal(SIGFPE,  SIG_IGN);
+	    case 'b':
+		/* Prevent git from using ANSI color sequences.  */
+		ansi_colors = OFF;
+		break;
+
+	    case 'l':
+		/* Prevent git from using the last character on the
+		   screen.  */
+		use_last_screen_character = OFF;
+		break;
+
+	    case 'p':
+		/* End the list of gitps arguments.  If this option is
+		   given, the following arguments are to be passed to
+		   `ps(1)'.  */
+		goto done;
+
+	    case '?':
+		return 1;
+
+	    default:
+		fprintf(stderr, "%s: unknown error\n", program);
+		return 1;
+	}
+
+  done:
+    if (optind < argc)
+    {
+	no_of_arguments = argc - optind;
+	arguments = (char **)xmalloc(sizeof(char *) * (no_of_arguments + 1));
+
+	for (i = 0; i < no_of_arguments; i++)
+	    arguments[i] = argv[optind++];
+
+	arguments[i] = NULL;
+    }
+    else
+	arguments = NULL;
+
+    title_text = xmalloc(strlen(PRODUCT) + strlen(VERSION) + 64);
+    sprintf(title_text, " %s %s - Process Viewer/Killer", PRODUCT, VERSION);
+
+    tty_init(TTY_FULL_INPUT);
 
     common_configuration_init();
     use_section("[GITPS-Keys]");
@@ -691,32 +1005,34 @@ main(argc, argv)
     configuration_end();
 
     specific_configuration_init();
-
-    tty_get_size(&SCREEN_X, &SCREEN_Y);
-    tty_startup(UseLastScreenChar);
-
-
     use_section("[Setup]");
 
-    configuration_getvarinfo("TempDirectory", &data, 1, DO_SEEK);
-    TempDirectory = data ? tilde_expand(data) : "/tmp";
+    temporary_directory = getenv("TMPDIR");
 
-    AnsiColors         = get_flag_var("AnsiColors", OFF);
-    UseLastScreenChar  = get_flag_var("UseLastScreenChar", OFF);
-    StartupScrollStep  = get_int_var("StartupScrollStep", (SCREEN_Y - 3) / 2);
+    if (temporary_directory == NULL)
+	temporary_directory = "/tmp";
 
-    if (StartupScrollStep <= 0 || StartupScrollStep >= (SCREEN_Y - 3) - 1)
-	StartupScrollStep = (SCREEN_Y - 3) / 2;
+    if (ansi_colors == -1)
+	AnsiColors = get_flag_var("AnsiColors", OFF);
+    else
+	AnsiColors = ansi_colors;
 
-    scroll_step = StartupScrollStep;
+    if (use_last_screen_character)
+	UseLastScreenChar = get_flag_var("UseLastScreenChar",  OFF);
+    else
+	UseLastScreenChar = OFF;
+
+    tty_set_last_char_flag(UseLastScreenChar);
+
+    StartupScrollStep  = get_int_var("StartupScrollStep", (tty_lines - 3) / 2);
 
 
     use_section("[GITPS-Setup]");
 
-    GitPsModeHelp = get_string_var("GitPsModeHelp", "");
+    help = get_string_var("Help", "");
 
 
-    use_section(AnsiColors ? cSection : bwSection);
+    use_section(AnsiColors ? color_section : monochrome_section);
 
     get_colorset_var(PSColors, PSFields, PS_FIELDS);
 
@@ -732,61 +1048,58 @@ main(argc, argv)
     configuration_end();
 
 #ifndef HAVE_LONG_FILE_NAMES
-    fprintf(stderr, "%s: warning: your system doesn't support long file names.",
+    fprintf(stderr,
+	    "%s: warning: your system doesn't support long file names.",
 	    program);
 #endif /* !HAVE_LONG_FILE_NAMES */
 
-#ifdef HAVE_LINUX
-    if (LinuxConsole)
-	screen = xmalloc(4 + SCREEN_X * SCREEN_Y * 2);
-#endif  /* HAVE_LINUX */
+    stdout_log_name = xmalloc(32 + strlen(temporary_directory) + 1);
+    stderr_log_name = xmalloc(32 + strlen(temporary_directory) + 1);
 
-    stdout_log_name = xmalloc(32 + strlen(TempDirectory) + 1);
-    stderr_log_name = xmalloc(32 + strlen(TempDirectory) + 1);
-    sprintf(stdout_log_name, "%s/gitps.1.%d", TempDirectory, (int)pid);
-    sprintf(stderr_log_name, "%s/gitps.2.%d", TempDirectory, (int)pid);
+    srand(time(NULL));
+    r1 = 1 + (int) (100000000.0 * rand() / (RAND_MAX + 1.0));
+    r2 = 1 + (int) (100000000.0 * rand() / (RAND_MAX + 1.0));
 
-    global_buf  = xmalloc(SCREEN_X + 1);
+    sprintf(stdout_log_name, "%s/gitps.1.%d.%d",
+	    temporary_directory, (int)getpid(), r1);
+    sprintf(stderr_log_name, "%s/gitps.2.%d.%d",
+	    temporary_directory, (int)getpid(), r2);
 
-    header_text = xmalloc(SCREEN_X + 1);
+    if (ps(arguments) == 0)
+	return 1;
 
-    title_win  = window_init(0, 0,            1,            SCREEN_X);
-    header_win = window_init(0, 1,            1,            SCREEN_X);
-    screen_win = window_init(0, 2,            SCREEN_Y - 3, SCREEN_X);
-    status_win = window_init(0, SCREEN_Y - 1, 1,            SCREEN_X);
+    title_window  = window_init();
+    header_window = window_init();
+    processes_window = window_init();
+    status_window = window_init();
+
+    resize(0);
 
     tty_get_screen(screen);
     tty_set_mode(TTY_NONCANONIC);
     tty_defaults();
 
+    signal_handlers(ON);
+
     first_on_screen = current_process = 0;
 
-restart:
-
-    if (ps(argc > 1 ? argv[1] : NULL) == 0)
-    {
-	remove_log();
-	goto end;
-    }
+    tty_update_title(ps_cmd);
 
+  restart:
     stdout_log = fopen(stdout_log_name, "r");
     remove_log();
 
     if ((PID_index = get_PID_index(stdout_log)) == -1)
+    {
+	exit_code = 1;
 	goto end;
+    }
 
     free_ps_list();
     build_ps_list(stdout_log);
     fclose(stdout_log);
 
-    settitle();
-    setstatus((char *)NULL);
-    setsignal();
-    setheader();
-
-    current_process = min(current_process, processes - 1);
-
-    update_all();
+    refresh(0);
 
     while (1)
     {
@@ -796,7 +1109,6 @@ restart:
 	switch (key)
 	{
 	    case BUILTIN_previous_line:
-
 		need_update_all = need_update = 0;
 
 		while (repeat_count--)
@@ -829,7 +1141,6 @@ restart:
 		break;
 
 	    case BUILTIN_next_line:
-
 		need_update_all = need_update = 0;
 
 		while (repeat_count--)
@@ -839,12 +1150,12 @@ restart:
 		    else
 			break;
 
-		    if (current_process - first_on_screen >= SCREEN_Y - 3)
+		    if (current_process - first_on_screen >= tty_lines - 3)
 		    {
 			first_on_screen = min(first_on_screen +
 					      scroll_step,
 					      processes - 1 -
-					      (SCREEN_Y - 3) + 1);
+					      (tty_lines - 3) + 1);
 			need_update_all = 1;
 			continue;
 		    }
@@ -863,21 +1174,20 @@ restart:
 		break;
 
 	    case BUILTIN_scroll_down:
-
 		if (current_process == 0)
 		    break;
 
 		old_current_process = current_process;
 
-		if (current_process < SCREEN_Y - 3)
+		if (current_process < tty_lines - 3)
 		    current_process = first_on_screen = 0;
 		else
 		{
-		    current_process -= SCREEN_Y - 3;
-		    first_on_screen = max(0, first_on_screen - (SCREEN_Y - 3));
+		    current_process -= tty_lines - 3;
+		    first_on_screen = max(0, first_on_screen - (tty_lines-3));
 		}
 
-		if (processes > SCREEN_Y - 3)
+		if (processes > tty_lines - 3)
 		    update_all();
 		else
 		{
@@ -888,29 +1198,28 @@ restart:
 		break;
 
 	    case BUILTIN_scroll_up:
-
 		if (current_process == processes - 1)
 		    break;
 
 		old_current_process = current_process;
 
-		if (processes - 1 - first_on_screen < SCREEN_Y - 3)
+		if (processes - 1 - first_on_screen < tty_lines - 3)
 		    current_process = processes - 1;
 		else
-		    if (processes - 1 - current_process < SCREEN_Y - 3)
+		    if (processes - 1 - current_process < tty_lines - 3)
 		    {
 			current_process = processes - 1;
-			first_on_screen = processes - 1 - (SCREEN_Y - 3) + 1;
+			first_on_screen = processes - 1 - (tty_lines - 3) + 1;
 		    }
 		    else
 		    {
-			current_process += SCREEN_Y - 3;
-			first_on_screen = min(first_on_screen + SCREEN_Y - 3,
+			current_process += tty_lines - 3;
+			first_on_screen = min(first_on_screen + tty_lines - 3,
 					      (processes - 1) -
-					      (SCREEN_Y - 3) + 1);
+					      (tty_lines - 3) + 1);
 		    }
 
-		if (processes > SCREEN_Y - 3)
+		if (processes > tty_lines - 3)
 		    update_all();
 		else
 		{
@@ -921,7 +1230,6 @@ restart:
 		break;
 
 	    case BUILTIN_beginning_of_list:
-
 		if (current_process == 0)
 		    break;
 
@@ -930,71 +1238,68 @@ restart:
 		break;
 
 	    case BUILTIN_end_of_list:
-
 		if (current_process == processes - 1)
 		    break;
 
 		current_process = processes - 1;
-		first_on_screen = max(0, (processes - 1) - (SCREEN_Y - 3) + 1);
+		first_on_screen = max(0, (processes-1) - (tty_lines-3) + 1);
 		update_all();
 		break;
 
 	    case BUILTIN_next_signal:
-
 		signal_type++;
-		signal_type %= sizeof(signals) / sizeof(struct SIGNAL);
-		setsignal();
+		signal_type %= sizeof(sigdesc) / sizeof(xsignal_t);
+		set_signal();
 		break;
 
-	    case BUILTIN_SIGHUP : signal_type =  0; setsignal(); break;
-	    case BUILTIN_SIGINT : signal_type =  1; setsignal(); break;
-	    case BUILTIN_SIGQUIT: signal_type =  2; setsignal(); break;
-	    case BUILTIN_SIGILL : signal_type =  3; setsignal(); break;
-	    case BUILTIN_SIGFPE : signal_type =  4; setsignal(); break;
-	    case BUILTIN_SIGKILL: signal_type =  5; setsignal(); break;
-	    case BUILTIN_SIGUSR1: signal_type =  6; setsignal(); break;
-	    case BUILTIN_SIGSEGV: signal_type =  7; setsignal(); break;
-	    case BUILTIN_SIGUSR2: signal_type =  8; setsignal(); break;
-	    case BUILTIN_SIGPIPE: signal_type =  9; setsignal(); break;
-	    case BUILTIN_SIGALRM: signal_type = 10; setsignal(); break;
-	    case BUILTIN_SIGTERM: signal_type = 11; setsignal(); break;
-	    case BUILTIN_SIGCHLD: signal_type = 12; setsignal(); break;
-	    case BUILTIN_SIGCONT: signal_type = 13; setsignal(); break;
+	    case BUILTIN_SIGHUP:  signal_type =  0; set_signal(); break;
+	    case BUILTIN_SIGINT:  signal_type =  1; set_signal(); break;
+	    case BUILTIN_SIGQUIT: signal_type =  2; set_signal(); break;
+	    case BUILTIN_SIGILL:  signal_type =  3; set_signal(); break;
+	    case BUILTIN_SIGFPE:  signal_type =  4; set_signal(); break;
+	    case BUILTIN_SIGKILL: signal_type =  5; set_signal(); break;
+	    case BUILTIN_SIGUSR1: signal_type =  6; set_signal(); break;
+	    case BUILTIN_SIGSEGV: signal_type =  7; set_signal(); break;
+	    case BUILTIN_SIGUSR2: signal_type =  8; set_signal(); break;
+	    case BUILTIN_SIGPIPE: signal_type =  9; set_signal(); break;
+	    case BUILTIN_SIGALRM: signal_type = 10; set_signal(); break;
+	    case BUILTIN_SIGTERM: signal_type = 11; set_signal(); break;
+	    case BUILTIN_SIGCHLD: signal_type = 12; set_signal(); break;
+	    case BUILTIN_SIGCONT: signal_type = 13; set_signal(); break;
 
 	    case BUILTIN_hard_refresh:
-
 		tty_touch();
 
 	    case BUILTIN_refresh:
-
+		ps(arguments);
 		goto restart;
 
 	    case BUILTIN_exit:
-
 		goto end;
 
 	    case BUILTIN_kill_process:
-
 		if (!kill_process(current_process))
 		{
+		    int e = errno;
+
 		    tty_beep();
-		    memset(global_buf, ' ', SCREEN_X);
-		    tmp = xmalloc(16 + strlen((errno==EPERM)?no_perm:no_proc));
-		    sprintf(tmp, "Error: %s", (errno==EPERM)?no_perm:no_proc);
-		    setstatus(tmp);
+		    memset(global_buf, ' ', tty_columns);
+		    tmp = xmalloc(16 + strlen((e == EPERM) ? no_perm:no_proc));
+		    sprintf(tmp, "Error: %s", (e == EPERM) ? no_perm:no_proc);
+		    set_status(tmp);
 		    xfree(tmp);
 		    errno = 0;
 		    tty_get_key(NULL);
-		    setstatus((char *)NULL);
-		    setsignal();
+		    set_status((char *)NULL);
+		    set_signal();
 		}
 		break;
 	}
     }
 
   end:
-
     remove_log();
-    tty_exit(screen);
-    return 0;
+    tty_set_mode(TTY_CANONIC);
+    tty_end(screen);
+    return exit_code;
 }
