@@ -1,6 +1,6 @@
 /* gitview.c -- A hex/ascii file viewer.  */
 
-/* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
+/* Copyright (C) 1993-1998 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Written by Tudor Hulubei and Andrei Pitis.  */
-
+/* $Id: gitview.c,v 1.5 1998/03/10 05:12:12 tudor Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -49,8 +49,10 @@
 #include "stdc.h"
 #include "xstring.h"
 #include "xmalloc.h"
+#include "getopt.h"
 #include "xio.h"
 #include "window.h"
+#include "signals.h"
 #include "configure.h"
 #include "tty.h"
 #include "misc.h"
@@ -93,9 +95,6 @@ char built_in[BUILTIN_OPERATIONS][MAX_BUILTIN_NAME] =
 };
 
 
-int SCREEN_X;
-int SCREEN_Y;
-
 #define VIEWER_FIELDS 12
 
 static char *ViewerFields[VIEWER_FIELDS] =
@@ -181,23 +180,21 @@ int AnsiColors = OFF;
 
 
 char *home;
-pid_t pid;
 char *program;
 char *screen;
+char *filename;
+int cnt, size;
+char offset[16];
 char *header_text;
 int  UseLastScreenChar;
-char *global_buf = NULL;
-char cSection[]  = "[GITVIEW-Color]";
-char bwSection[] = "[GITVIEW-Monochrome]";
+char *global_buf;
+char color_section[]  = "[GITVIEW-Color]";
+char monochrome_section[] = "[GITVIEW-Monochrome]";
 int fd, regular_file;
 int current_line, lines;
-window_t *title_win, *header_win, *screen_win, *status_win;
-#ifdef HAVE_GCC
-static char title_text[] = " "PRODUCT" "VERSION" - Hex/Ascii File Viewer";
-#else
-static char title_text[] = " GNU Interactive Tools 4.3.16 - Hex/Ascii File Viewer";
-#endif /* !HAVE_GCC */
-static char *GitViewModeHelp;
+window_t *title_window, *header_window, *file_window, *status_window;
+static char *title_text;
+static char *help;
 static char info_txt[] =
     "   Offset    00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F        \
 Ascii       ";
@@ -223,45 +220,47 @@ file_length()
 
 
 void
-settitle()
+set_title()
 {
-    memset(global_buf, ' ', SCREEN_X);
-    memcpy(global_buf, title_text, strlen(title_text));
+    memset(global_buf, ' ', tty_columns);
+    memcpy(global_buf, title_text,
+	   min(tty_columns, (int)strlen(title_text)));
 
     tty_colors(TitleBrightness, TitleForeground, TitleBackground);
 
-    window_goto(title_win, 0, 0);
-    window_puts(global_buf, SCREEN_X);
+    window_goto(title_window, 0, 0);
+    window_puts(title_window, global_buf, tty_columns);
 }
 
 
 void
-setheader()
+set_header()
 {
-    memset(global_buf, ' ', SCREEN_X);
-    memcpy(global_buf, header_text, min(strlen(header_text), SCREEN_X));
+    memset(global_buf, ' ', tty_columns);
+    memcpy(global_buf, header_text,
+	   min(tty_columns, (int)strlen(header_text)));
 
     tty_colors(HeaderBrightness, HeaderForeground, HeaderBackground);
 
-    window_goto(header_win, 0, 0);
-    window_puts(global_buf, SCREEN_X);
+    window_goto(header_window, 0, 0);
+    window_puts(header_window, global_buf, tty_columns);
 }
 
 
 void
-setstatus()
+set_status()
 {
-    memset(global_buf, ' ', SCREEN_X);
-    memcpy(global_buf, GitViewModeHelp, strlen(GitViewModeHelp));
+    memset(global_buf, ' ', tty_columns);
+    memcpy(global_buf, help, min(tty_columns, (int)strlen(help)));
 
     tty_colors(StatusBrightness, StatusForeground, StatusBackground);
 
-    window_goto(status_win, 0, 0);
+    window_goto(status_window, 0, 0);
 
     if (UseLastScreenChar)
-	window_puts(global_buf, SCREEN_X);
+	window_puts(status_window, global_buf, tty_columns);
     else
-	window_puts(global_buf, SCREEN_X - 1);
+	window_puts(status_window, global_buf, tty_columns - 1);
 }
 
 
@@ -269,7 +268,7 @@ char
 char_to_print(c)
     char c;
 {
-    return is_print(c) ? c : '.';
+    return isprint(c) ? c : '.';
 }
 
 
@@ -278,37 +277,47 @@ update_line(line)
     int line;
 {
     int r;
-    unsigned char ln[16];
+    unsigned char buf[16];
+    char *line_string = xmalloc(max(tty_columns, 80 + 1));
 
-    memset(ln, 0, 16);
-    memset(global_buf, ' ', SCREEN_X);
-    lseek(fd, line * 16, SEEK_SET);
+    memset(line_string, ' ', tty_columns);
+    memset(buf, '\0', sizeof(buf));
+    lseek(fd, line * sizeof(buf), SEEK_SET);
 
-    if ((r = read(fd, ln, 16)))
-	sprintf(global_buf, "  %07X0   %02X %02X %02X %02X %02X %02X %02X\
- %02X %02X %02X %02X %02X %02X %02X %02X %02X  %c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c  ",
+    if ((r = read(fd, buf, sizeof(buf))))
+    {
+	sprintf(line_string, "  %07X0   %02X %02X %02X %02X %02X %02X %02X\
+ %02X %02X %02X %02X %02X %02X %02X %02X %02X  \
+%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c  ",
 		line,
-		ln[0], ln[1], ln[2],  ln[3],  ln[4],  ln[5],  ln[6],  ln[7],
-		ln[8], ln[9], ln[10], ln[11], ln[12], ln[13], ln[14], ln[15],
-		char_to_print(ln[0]),  char_to_print(ln[1]),
-		char_to_print(ln[2]),  char_to_print(ln[3]),
-		char_to_print(ln[4]),  char_to_print(ln[5]),
-		char_to_print(ln[6]),  char_to_print(ln[7]),
-		char_to_print(ln[8]),  char_to_print(ln[9]),
-		char_to_print(ln[10]), char_to_print(ln[11]),
-		char_to_print(ln[12]), char_to_print(ln[13]),
-		char_to_print(ln[14]), char_to_print(ln[15]));
-
-    if (r < 0)
+		buf[0],  buf[1],  buf[2],  buf[3],
+		buf[4],  buf[5],  buf[6],  buf[7],
+		buf[8],  buf[9],  buf[10], buf[11],
+		buf[12], buf[13], buf[14], buf[15],
+		char_to_print(buf[0]),  char_to_print(buf[1]),
+		char_to_print(buf[2]),  char_to_print(buf[3]),
+		char_to_print(buf[4]),  char_to_print(buf[5]),
+		char_to_print(buf[6]),  char_to_print(buf[7]),
+		char_to_print(buf[8]),  char_to_print(buf[9]),
+		char_to_print(buf[10]), char_to_print(buf[11]),
+		char_to_print(buf[12]), char_to_print(buf[13]),
+		char_to_print(buf[14]), char_to_print(buf[15]));
+    }
+    else
+    {
 	r = 0;
+	*line_string = '\0';
+    }
 
     if (r != 16)
     {
-	memset(global_buf + 13 + r * 3, ' ', (16 - r) * 3 - 1);
-	memset(global_buf + 13 + 16 * 3 - 1 + 2 + r, ' ', 16 - r + 2);
+	memset(line_string + 13 + r * 3, ' ', (16 - r) * 3 - 1);
+	memset(line_string + 13 + 16 * 3 - 1 + 2 + r, ' ', 16 - r + 2);
     }
 
-    window_puts((char *)global_buf, SCREEN_X);
+    line_string[strlen(line_string)] = ' ';
+    window_puts(file_window, line_string, tty_columns);
+    xfree(line_string);
 }
 
 
@@ -321,7 +330,7 @@ update_all()
 
     for (i = current_line; i < current_line + 16; i++)
     {
-	window_goto(screen_win, 3 + i - current_line, 0);
+	window_goto(file_window, 3 + i - current_line, 0);
 	update_line(i);
     }
 }
@@ -330,7 +339,7 @@ update_all()
 void
 clean_up()
 {
-    tty_exit(NULL);
+    tty_end(NULL);
 }
 
 
@@ -344,14 +353,6 @@ fatal(postmsg)
 }
 
 
-RETSIGTYPE
-panic(signum)
-    int signum;
-{
-    fatal_signal(signum);
-}
-
-
 int
 read_keys(keys)
     int keys;
@@ -417,80 +418,244 @@ read_keys(keys)
 }
 
 
+void
+resize(resize_required)
+    int resize_required;
+{
+    int display_title = OFF;
+    int display_header = OFF;
+    int display_file = OFF;
+    int old_tty_lines = tty_lines;
+    int old_tty_columns = tty_columns;
+
+    tty_resize();
+
+    /* Don't resize, unless absolutely necessary.  */
+    if (!resize_required)
+	if (tty_lines == old_tty_lines && tty_columns == old_tty_columns)
+	    return;
+
+#ifdef HAVE_LINUX
+    if (LinuxConsole)
+	screen = xrealloc(screen, 4 + tty_columns * tty_lines * 2);
+#endif  /* HAVE_LINUX */
+
+    /* Watch out for special cases (tty_lines < 7) because some
+     * components will no longer fit.
+     *
+     * The title needs one line.  The panels need four.  There is one
+     * more line for the input line and one for the status.
+     *
+     * Cases (lines available):
+     *      1 line:	the status
+     *      2 lines:	title + status
+     *      3 lines:	title + header + status
+     *	 >= 4 lines:	everything
+     *
+     * FIXME: we need to handle the tty_columns case.
+     */
+
+    global_buf  = xrealloc(global_buf, tty_columns + 1);
+    header_text = xrealloc(header_text, strlen(filename) + 10);
+
+    window_resize(status_window, 0, tty_lines - 1, 1, tty_columns);
+
+    if (tty_lines >= 2)
+	display_title = ON;
+
+    if (tty_lines >= 3)
+	display_header = ON;
+
+    if (tty_lines >= 4)
+	display_file = ON;
+
+    window_resize(title_window, 0, 0, display_title ? 1 : 0, tty_columns);
+    window_resize(header_window, 0, 1, display_header ? 1 : 0, tty_columns);
+    window_resize(file_window, 0, 2,
+		  display_file ? (tty_lines - 2) : 0, tty_columns);
+}
+
+
+/*
+ * Resize (if necessary) and then refresh all gitview's components.
+ */
+
+void
+refresh(signum)
+    int signum;
+{
+    resize(0);
+
+    if (signum == SIGCONT)
+    {
+	/* We were suspended.  Switch back to noncanonical mode.  */
+	tty_set_mode(TTY_NONCANONIC);
+	tty_defaults();
+    }
+
+    tty_colors(ScreenBrightness, ScreenForeground, ScreenBackground);
+    tty_fill();
+
+    window_goto(file_window, 1, 0);
+    window_puts(file_window, info_txt, sizeof(info_txt) - 1);
+    window_goto(file_window, 2, 0);
+    window_puts(file_window, line_txt, sizeof(line_txt) - 1);
+
+    size = file_length();
+    lines = size / 16 + (size % 16 ? 1 : 0);
+    current_line = min(current_line, (lines / 16) * 16);
+
+    window_goto(file_window, 20, 0);
+    window_puts(file_window, seek_txt, sizeof(seek_txt) - 1);
+    window_goto(file_window, 20, sizeof(seek_txt) - 1);
+    window_puts(file_window, offset, cnt);
+
+    set_title();
+    set_header();
+    set_status();
+    update_all();
+
+    window_goto(file_window, 20, sizeof(seek_txt) - 1 + cnt);
+
+    tty_update();
+
+    if (signum == SIGCONT)
+	tty_update_title(header_text);
+}
+
+
+void
+hide()
+{
+    tty_set_mode(TTY_CANONIC);
+    tty_defaults();
+    tty_put_screen(screen);
+}
+
+
+void
+clock_refresh()
+{
+}
+
+
+void
+usage()
+{
+    printf("usage: %s [-hvicbl] file\n", program);
+    printf(" -h         print this help message\n");
+    printf(" -v         print the version number\n");
+    printf(" -i         print the installation directory\n");
+    printf(" -c         use ANSI colors\n");
+    printf(" -b         don't use ANSI colors\n");
+    printf(" -l         don't use the last screen character\n");
+}
+
+
 int
 main(argc, argv)
     int argc;
     char *argv[];
 {
-    char c;
+    int key;
     struct stat s;
     tty_key_t *ks;
-    char offset[16];
-    int cnt = 0, size;
-    int key, first_time = 1;
     int keys, repeat_count, need_update;
+    int c, ansi_colors = -1, use_last_screen_character = ON;
 
 
+    /* Make sure we don't get signals before we are ready to handle
+       them.  */
+    signals_init();
+
     program = argv[0];
 
     home = getenv("HOME");
     if (home == NULL)
 	home = ".";
 
-    get_tty_name();
+    compute_directories();
     get_login_name();
 
-    tty_name_len = strlen(tty_name);
+    if (getenv("COLORTERM") != NULL)
+	ansi_colors = ON;
 
-    tty_get_capabilities();
-    tty_kbdinit(TTY_RESTRICTED_INPUT);
+    /* Parse the command line.  */
+    while ((c = getopt(argc, argv, "hvicblp")) != -1)
+	switch (c)
+	{
+	    case 'h':
+		/* Help request.  */
+		usage();
+		return 0;
 
-    signal(SIGTERM, panic);
-    signal(SIGINT , panic);
-    signal(SIGQUIT, panic);
-    signal(SIGSEGV, panic);
-    signal(SIGHUP,  panic);
+	    case 'v':
+		/* Version number request.  */
+		printf("%s %s\n", PRODUCT, VERSION);
+		return 0;
 
-    signal(SIGILL,  SIG_IGN);
-    signal(SIGTRAP, SIG_IGN);
-    signal(SIGABRT, SIG_IGN);
-    signal(SIGUSR1, SIG_IGN);
-    signal(SIGUSR2, SIG_IGN);
+	    case 'i':
+		/* Installation directory request.  */
+		printf("%s is installed in %s.\n", PRODUCT, PREFIX);
+		return 0;
 
-#ifdef SIGTSTP
-    signal(SIGTSTP, SIG_IGN);
-#endif
+	    case 'c':
+		/* Force git to use ANSI color sequences.  */
+		ansi_colors = ON;
+		break;
 
-#ifdef SIGCONT
-    signal(SIGCONT, SIG_IGN);
-#endif
+	    case 'b':
+		/* Prevent git from using ANSI color sequences.  */
+		ansi_colors = OFF;
+		break;
 
-    signal(SIGALRM, SIG_IGN);
-    signal(SIGPIPE, SIG_IGN);
-    signal(SIGFPE,  SIG_IGN);
+	    case 'l':
+		/* Prevent git from using the last character on the
+		   screen.  */
+		use_last_screen_character = OFF;
+		break;
 
-    if (argc != 2)
+	    case '?':
+		return 1;
+
+	    default:
+		fprintf(stderr, "%s: unknown error\n", program);
+		return 1;
+	}
+
+    if (optind < argc)
+	filename = xstrdup(argv[optind++]);
+    else
     {
-	fprintf(stderr, "usage: gitview filename\n");
-	exit(1);
+	usage();
+	return 1;
     }
 
-    xstat(argv[1], &s);
+    if (optind < argc)
+	fprintf(stderr, "%s: warning: invalid extra options ignored\n",
+		program);
+
+    title_text = xmalloc(strlen(PRODUCT) + strlen(VERSION) + 64);
+    sprintf(title_text, " %s %s - Hex/Ascii File Viewer", PRODUCT, VERSION);
+
+    tty_init(TTY_RESTRICTED_INPUT);
+
+    xstat(filename, &s);
 
 /*
     if (!(S_ISREG(s.st_mode) || S_ISBLK(s.st_mode)))
     {
 	fprintf(stderr, "%s: %s is neither regular file nor block device.\n",
-		program, argv[1]);
-	exit(1);
+		program, filename);
+	return 1;
     }
 */
-    fd = open(argv[1], O_RDONLY);
+    fd = open(filename, O_RDONLY);
 
     if (fd == -1)
     {
-	fprintf(stderr, "%s: cannot open file %s.\n", program, argv[1]);
-	exit(1);
+	fprintf(stderr, "%s: cannot open file %s.\n", program, filename);
+	return 1;
     }
 
     regular_file = S_ISREG(s.st_mode);
@@ -504,15 +669,24 @@ main(argc, argv)
 
     use_section("[Setup]");
 
-    AnsiColors         = get_flag_var("AnsiColors", OFF);
-    UseLastScreenChar  = get_flag_var("UseLastScreenChar", OFF);
+    if (ansi_colors == -1)
+	AnsiColors = get_flag_var("AnsiColors", OFF);
+    else
+	AnsiColors = ansi_colors;
+
+    if (use_last_screen_character)
+	UseLastScreenChar = get_flag_var("UseLastScreenChar",  OFF);
+    else
+	UseLastScreenChar = OFF;
+
+    tty_set_last_char_flag(UseLastScreenChar);
 
 
     use_section("[GITVIEW-Setup]");
 
-    GitViewModeHelp = get_string_var("GitViewModeHelp", "");
+    help = get_string_var("Help", "");
 
-    use_section(AnsiColors ? cSection : bwSection);
+    use_section(AnsiColors ? color_section : monochrome_section);
 
     get_colorset_var(ViewerColors, ViewerFields, VIEWER_FIELDS);
 
@@ -527,75 +701,28 @@ main(argc, argv)
 
     configuration_end();
 
-    tty_get_size(&SCREEN_X, &SCREEN_Y);
-    tty_startup(UseLastScreenChar);
+    title_window  = window_init();
+    header_window = window_init();
+    file_window = window_init();
+    status_window = window_init();
 
-    if (SCREEN_Y < 24)
-    {
-	SCREEN_Y = 24;
-	fprintf(stderr, "%s: WARNING: can't use less than 24 columns.\n",
-		program);
-    }
-
-#ifdef HAVE_LINUX
-    if (LinuxConsole)
-	screen = xmalloc(4 + SCREEN_X * SCREEN_Y * 2);
-#endif  /* HAVE_LINUX */
-
-    global_buf  = xmalloc(SCREEN_X + 1);
-
-    header_text = xmalloc(strlen(argv[1]) + 10);
-    sprintf(header_text, " File: %s", argv[1]);
-
-    title_win  = window_init(0, 0,            1,            SCREEN_X);
-    header_win = window_init(0, 1,            1,            SCREEN_X);
-    screen_win = window_init(0, 2,            SCREEN_Y - 2, SCREEN_X);
-    status_win = window_init(0, SCREEN_Y - 1, 1,            SCREEN_X);
+    resize(0);
 
     tty_get_screen(screen);
     tty_set_mode(TTY_NONCANONIC);
     tty_defaults();
 
+    signal_handlers(ON);
+
     offset[cnt]  = 0;
     current_line = 0;
 
+    sprintf(header_text, " File: %s", filename);
+
+    tty_update_title(header_text);
 
   restart:
-
-    tty_colors(ScreenBrightness, ScreenForeground, ScreenBackground);
-
-    if (first_time)
-    {
-	tty_fill();
-	first_time = 0;
-    }
-
-    settitle();
-    setstatus();
-    setheader();
-
-    tty_colors(ScreenBrightness, ScreenForeground, ScreenBackground);
-
-    window_goto(screen_win, 1, 0);
-    window_puts(info_txt, sizeof(info_txt) - 1);
-    window_goto(screen_win, 2, 0);
-    window_puts(line_txt, sizeof(line_txt) - 1);
-
-    size = file_length();
-    lines = size / 16 + (size % 16 ? 1 : 0);
-
-    current_line = min(current_line, (lines / 16) * 16);
-
-    update_all();
-
-    window_goto(screen_win, 20, 0);
-    window_puts(seek_txt, sizeof(seek_txt) - 1);
-    window_goto(screen_win, 20, sizeof(seek_txt) - 1);
-    window_puts(offset, cnt);
-
-    window_goto(screen_win, 20, sizeof(seek_txt) - 1 + cnt);
-
-    first_time = 1;
+    refresh(0);
 
     while (1)
     {
@@ -615,7 +742,6 @@ main(argc, argv)
 		goto end;
 
 	    case BUILTIN_previous_line:
-
 		need_update = 0;
 
 		while (repeat_count--)
@@ -632,7 +758,6 @@ main(argc, argv)
 		break;
 
 	    case BUILTIN_next_line:
-
 		need_update = 0;
 
 		while (repeat_count--)
@@ -649,7 +774,6 @@ main(argc, argv)
 		break;
 
 	    case BUILTIN_scroll_down:
-
 		if (current_line == 0)
 		    break;
 
@@ -658,7 +782,6 @@ main(argc, argv)
 		break;
 
 	    case BUILTIN_scroll_up:
-
 		if (current_line >= lines - 16)
 		    break;
 
@@ -667,7 +790,6 @@ main(argc, argv)
 		break;
 
 	    case BUILTIN_beginning_of_file:
-
 		if (current_line)
 		{
 		    current_line = 0;
@@ -677,7 +799,6 @@ main(argc, argv)
 		break;
 
 	    case BUILTIN_end_of_file:
-
 		if (regular_file && current_line < lines - 16)
 		{
 		    current_line = lines - 16;
@@ -687,11 +808,7 @@ main(argc, argv)
 		break;
 
 	    case BUILTIN_hard_refresh:
-
-		first_time = 1;
-
 	    case BUILTIN_refresh:
-
 		goto restart;
 
 	    case '0': case '1': case '2': case '3': case '4': case '5':
@@ -700,34 +817,33 @@ main(argc, argv)
 	    case 'c': case 'd': case 'e': case 'f':
 		if (cnt < 8)
 		{
-		    window_goto(screen_win, 20, strlen(seek_txt) + cnt);
-		    c = (char)key;
-		    window_putc(c);
-		    offset[cnt++] = c;
+		    char tmp;
+		    window_goto(file_window, 20, strlen(seek_txt) + cnt);
+		    tmp = (char)key;
+		    window_putc(file_window, tmp);
+		    offset[cnt++] = tmp;
 		}
 		else
 		    tty_beep();
 		break;
 
 	    case BUILTIN_backspace:
-
 		if (cnt)
 		    cnt--;
 
-		window_goto(screen_win, 20, strlen(seek_txt) + cnt);
-		window_putc(' ');
+		window_goto(file_window, 20, strlen(seek_txt) + cnt);
+		window_putc(file_window, ' ');
 		break;
 
 	    case BUILTIN_action:
-
 		if (cnt == 0)
 		    tty_beep();
 		else
 		{
 		    offset[cnt] = 0;
 		    sscanf(offset, "%x", &cnt);
-		    window_goto(screen_win, 20, strlen(seek_txt));
-		    window_puts("        ", 8);
+		    window_goto(file_window, 20, strlen(seek_txt));
+		    window_puts(file_window, "        ", 8);
 
 		    if (cnt < 0)
 			cnt = 0;
@@ -749,11 +865,11 @@ main(argc, argv)
 		break;
 	}
 
-	window_goto(screen_win, 20, strlen(seek_txt) + cnt);
+	window_goto(file_window, 20, strlen(seek_txt) + cnt);
     }
 
   end:
-
-    tty_exit(screen);
+    tty_set_mode(TTY_CANONIC);
+    tty_end(screen);
     return 0;
 }
