@@ -1,7 +1,7 @@
 /* gitwipe.c -- An utility that deletes the contents of a file in order to
    make it impossible for someone to recover it.  */
 
-/* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
+/* Copyright (C) 1993-1998 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Written by Tudor Hulubei and Andrei Pitis.  */
-
+/* $Id: gitwipe.c,v 1.3 1998/03/10 05:12:12 tudor Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -82,9 +82,9 @@ int
 wipe(file)
     char *file;
 {
-    int fd, i, j;
+    int fd;
     unsigned char *buf;
-    size_t len, bytes_to_write;
+    size_t len, i, j, bytes_to_write;
 
     fd = open(file, O_RDWR);
 
@@ -114,7 +114,7 @@ wipe(file)
 	for (j = 0; j < bytes_to_write; j++)
 	    buf[j] = rand() % 0xFF;
 
-	if (write(fd, buf, bytes_to_write) != bytes_to_write)
+	if (write(fd, buf, bytes_to_write) != (int)bytes_to_write)
 	{
 	    fprintf(stderr, "%s: can't write to file %s.\n", program, file);
 	    return 1;
