@@ -1,6 +1,6 @@
 /* inputline.c -- Input line management functions.  */
 
-/* Copyright (C) 1993, 1994, 1995, 1996 Free Software Foundation, Inc.
+/* Copyright (C) 1993-1998 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Written by Tudor Hulubei and Andrei Pitis.  */
-
+/* $Id: inputline.c,v 1.3 1998/03/10 05:12:12 tudor Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -38,6 +38,7 @@
 #endif /* !HAVE_STDLIB_H */
 
 #include <ctype.h>
+#include <assert.h>
 
 #include "xstring.h"
 #include "xmalloc.h"
@@ -119,13 +120,10 @@ static int InputLineColors[INPUTLINE_FIELDS] =
 #define IL_EXCHANGE_POINT_AND_MARK		27
 
 
-#define IL_HORIZONTAL_SCROLL                    20
-
-
-#define IL_RESIZE(il_new_buffer_size)                   \
-{                                                       \
-    il->size   = il_new_buffer_size;                    \
-    il->buffer = xrealloc(il->buffer, il->size);        \
+#define IL_RESIZE(il_new_buffer_size)			\
+{							\
+    il->size = il_new_buffer_size;			\
+    il->buffer = xrealloc(il->buffer, il->size);	\
 }
 
 
@@ -225,28 +223,39 @@ il_free(some_il)
 }
 
 
+/*
+ * Compute the amount of scrolling when the point is at the end of the
+ * input line.  It should be about 25% of the dynamic part of the
+ * input line, but if the input line shrinks (as a result of a
+ * SIGWINCH) it should not become less than 1.
+ */
+
+static size_t
+il_compute_scroll()
+{
+    return max((il->columns - il->static_length) / 4,  1);
+}
+
+
 /*
  * The input_line constructor.
  */
 
 void
-il_init(_columns, _begin_y)
-    size_t _columns, _begin_y;
+il_init()
 {
     char *data;
 
     il = (input_line_t *)xmalloc(sizeof(input_line_t));
 
-    il->echo           = 1;
-    il->error          = 0;
-    il->buffer         = NULL;
-    il->kill_ring      = NULL;
-    il->columns        = _columns;
-    il->last_operation = IL_NO_OPERATION;
-
+    il->echo = 1;
+    il->error = 0;
+    il->buffer = NULL;
+    il->kill_ring = NULL;
+    il->columns = 0;
+    il->line = 0;
     il_reset_line();
-
-    il->win = window_init(0, _begin_y, 1, _columns);
+    il->window = window_init();
 
 
     use_section("[GIT-Setup]");
@@ -254,7 +263,7 @@ il_init(_columns, _begin_y)
     configuration_getvarinfo("HistoryFile", &data, 1, DO_SEEK);
     il->history_file = tilde_expand(data ? data : "~/.githistory");
 
-    use_section(AnsiColors ? cSection : bwSection);
+    use_section(AnsiColors ? color_section : monochrome_section);
 
     get_colorset_var(InputLineColors, InputLineFields, INPUTLINE_FIELDS);
 
@@ -275,13 +284,27 @@ void
 il_end()
 {
     write_history(il->history_file);
-    window_end(il->win);
+    window_end(il->window);
 
     il_free(il);
     il = NULL;
 }
 
 
+/*
+ * Resize the input line.
+ */
+
+void
+il_resize(_columns, _line)
+    int _columns, _line;
+{
+    il->columns = _columns;
+    il->line = _line;
+    window_resize(il->window, 0, _line, 1, _columns);
+}
+
+
 /*
  * Save the entire input_line structure.  Returns a pointer to the saved
  * structure.
@@ -315,11 +338,17 @@ void
 il_restore(saved_il)
     input_line_t *saved_il;
 {
+    size_t columns = il->columns;
+    size_t line = il->line;
+
     if (saved_il == NULL)
 	return;
 
     il_free(il);
     il = saved_il;
+
+    /* We need this in case we got a SIGWINCH.  */
+    il_resize(columns, line);
 }
 
 
@@ -519,9 +548,9 @@ il_end_of_line()
 
 void
 il_insert_char(c)
-    char c;
+    int c;
 {
-    if (!is_print(c))
+    if (!isprint(c))
 	return;
 
     if (il->length + 1 >= il->size)
@@ -595,8 +624,8 @@ il_backward_delete_char()
 void
 il_kill_word()
 {
-    int end_of_word;
-    int old_mark = il->mark;
+    size_t end_of_word;
+    size_t old_mark = il->mark;
 
     il_set_mark();
     il_forward_word();
@@ -624,8 +653,8 @@ il_kill_word()
 void
 il_backward_kill_word()
 {
-    int old_mark  = il->mark;
-    int old_point = il->point;
+    size_t old_mark  = il->mark;
+    size_t old_point = il->point;
 
     il_set_mark();
     il_backward_word();
@@ -689,8 +718,8 @@ il_kill_line(store)
 void
 il_kill_to_beginning_of_line()
 {
-    int old_mark = (il->mark <= il->point) ? il->static_length :
-					     il->mark - il->point;
+    size_t old_mark = (il->mark <= il->point) ? il->static_length :
+						il->mark - il->point;
 
     il_set_mark();
     il_beginning_of_line();
@@ -708,7 +737,7 @@ il_kill_to_beginning_of_line()
 void
 il_kill_to_end_of_line()
 {
-    int old_mark = il->mark;
+    size_t old_mark = il->mark;
 
     il_set_mark();
     il_end_of_line();
@@ -765,7 +794,8 @@ il_downcase_word()
 {
     if (il->point < il->length)
     {
-	int i, previous_point = il->point;
+	size_t i;
+	size_t previous_point = il->point;
 
 	/* Move to the end of the word.  */
 	il_forward_word();
@@ -787,7 +817,8 @@ il_upcase_word()
 {
     if (il->point < il->length)
     {
-	int i, previous_point = il->point;
+	size_t i;
+	size_t previous_point = il->point;
 
 	/* Move to the end of the word.  */
 	il_forward_word();
@@ -810,7 +841,9 @@ il_capitalize_word()
 {
     if (il->point < il->length)
     {
-	int i, first = 1, previous_point = il->point;
+	size_t i;
+	int first = 1;
+	size_t previous_point = il->point;
 
 	/* Move to the end of the word.  */
 	il_forward_word();
@@ -843,8 +876,7 @@ il_set_static_text(static_text)
 {
     size_t len;
 
-    if (static_text == NULL)
-	return;
+    assert(static_text);
 
     len = strlen(static_text);
 
@@ -860,8 +892,9 @@ il_set_static_text(static_text)
 
     memcpy(il->buffer, static_text, len);
 
-    il->length = (il->static_length = len) + il->dynamic_length;
+    toprintable(il->buffer, len);
 
+    il->length = (il->static_length = len) + il->dynamic_length;
     il->last_operation = IL_SET_STATIC_TEXT;
 }
 
@@ -890,6 +923,8 @@ il_insert_text(text)
 
     memcpy(il->buffer + il->point, text, len);
 
+    toprintable(il->buffer + il->point, len);
+
     il->point          += len;
     il->length         += len;
     il->dynamic_length += len;
@@ -898,6 +933,43 @@ il_insert_text(text)
 }
 
 
+/*
+ * Temporary hide the static part of the input line.  This is only used
+ * when the visible part of the input line becomes smaller than the
+ * static size.
+ */
+
+static int
+il_hide_static()
+{
+    int normal_static_length = il->static_length;
+
+    il->buffer += il->static_length;
+    il->length -= il->static_length;
+    il->point -= il->static_length;
+    il->mark -= il->static_length;
+    il->static_length = 0;
+
+    return normal_static_length;
+}
+
+
+/*
+ * Restore the static part of the input line.
+ */
+
+static void
+il_restore_static(normal_static_length)
+    int normal_static_length;
+{
+    il->static_length = normal_static_length;
+    il->buffer -= il->static_length;
+    il->length += il->static_length;
+    il->point += il->static_length;
+    il->mark += il->static_length;
+}
+
+
 /*
  * Update the point.
  */
@@ -905,17 +977,28 @@ il_insert_text(text)
 void
 il_update_point()
 {
+    int scroll;
     size_t len;
+    size_t normal_static_length = 0;
+    int il_too_small = il->columns < il->static_length + 3;
 
     tty_colors(il->error ? InputLineErrorBrightness : InputLineBrightness,
 	       il->error ? InputLineErrorForeground : InputLineForeground,
 	       il->error ? InputLineErrorBackground : InputLineBackground);
 
-    len = (il->point >= il->columns) ?
-	  il->point - il->columns + 1 + (IL_HORIZONTAL_SCROLL - 1) -
-	  (il->point % IL_HORIZONTAL_SCROLL) : 0;
+    if (il_too_small)
+	normal_static_length = il_hide_static();
 
-    window_goto(il->win, 0, il->point - len);
+    scroll = il_compute_scroll();
+
+    len = ((il->point >= il->columns) ?
+	   il->point - il->columns + 1 +
+	   (scroll - 1) - ((il->point - il->columns) % scroll) : 0);
+
+    window_goto(il->window, 0, il->point - len);
+
+    if (il_too_small)
+	il_restore_static(normal_static_length);
 }
 
 
@@ -926,10 +1009,12 @@ il_update_point()
 void
 il_update()
 {
+    int scroll;
     char *temp;
     unsigned len;
     tty_status_t status;
-
+    size_t normal_static_length = 0;
+    int il_too_small = il->columns < il->static_length + 3;
 
     tty_save(&status);
 
@@ -937,12 +1022,16 @@ il_update()
 	       il->error ? InputLineErrorForeground : InputLineForeground,
 	       il->error ? InputLineErrorBackground : InputLineBackground);
 
-    len = (il->point >= il->columns) ?
-	  il->point - il->columns + 1 + (IL_HORIZONTAL_SCROLL - 1) -
-	  (il->point % IL_HORIZONTAL_SCROLL) : 0;
+    if (il_too_small)
+	normal_static_length = il_hide_static();
+
+    scroll = il_compute_scroll();
+
+    len = ((il->point >= il->columns) ?
+	   il->point - il->columns + 1 +
+	   (scroll - 1) - ((il->point - il->columns) % scroll) : 0);
 
     temp = xmalloc(il->columns);
-
     memset(temp, ' ', il->columns);
 
     if (il->echo)
@@ -954,16 +1043,22 @@ il_update()
 	       min(il->length   - il->static_length - len,
 		   il->columns  - il->static_length));
 
-    window_goto(il->win, 0, 0);
-    window_puts(il->buffer, il->static_length);
-    window_puts(temp, il->columns - il->static_length);
+    window_goto(il->window, 0, 0);
+
+    if (!il_too_small)
+	window_puts(il->window, il->buffer, il->static_length);
+
+    window_puts(il->window, temp, il->columns - il->static_length);
 
     /* If we don't do this, the screen cursor will annoyingly jump to
        the left margin of the command line.  */
-    window_goto(il->win, 0, il->point - len);
+    window_goto(il->window, 0, il->point - len);
 
     window_update();
 
+    if (il_too_small)
+	il_restore_static(normal_static_length);
+
     xfree(temp);
     tty_restore(&status);
 }
@@ -1034,7 +1129,6 @@ il_history(dir)
     switch (dir)
     {
 	case IL_PREVIOUS:
-
 	    if (!browsing)
 	    {
 		browsing = 1;
@@ -1058,7 +1152,6 @@ il_history(dir)
 	    break;
 
 	case IL_NEXT:
-
 	    if (!browsing)
 	    {
 		browsing = 1;
@@ -1083,7 +1176,6 @@ il_history(dir)
 	    break;
 
 	case IL_RECORD:
-
 	    if (browsing)
 	    {
 		history_set_pos(last_history_position);
@@ -1105,7 +1197,6 @@ il_history(dir)
 	    break;
 
 	default:
-
 	    break;
     }
 }
