@@ -1,6 +1,6 @@
 /* misc.c -- Miscelaneous functions used in git/gitps/gitview.  */
 
-/* Copyright (C) 1993, 1994, 1995, 1996 Free Software Foundation, Inc.
+/* Copyright (C) 1993-1998 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Written by Tudor Hulubei and Andrei Pitis.  */
-
+/* $Id: misc.c,v 1.4 1998/03/10 05:12:12 tudor Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -32,7 +32,7 @@
 #endif /* !HAVE_STDLIB_H */
 
 #include <signal.h>
-
+#include <ctype.h>
 #include <sys/types.h>
 
 #ifdef HAVE_STDDEF_H
@@ -54,12 +54,11 @@
 
 
 static char CONFIGFILE_PREFIX[] = "/.gitrc.";
-static char term[] = TERMDIR;
 
-char  *tty_name;
-size_t tty_name_len;
-char  *login_name;
-size_t login_name_len;
+static char *termdir;
+static char *bindir;
+
+char *login_name;
 
 
 char *day_name[] =
@@ -81,6 +80,44 @@ file_type_info_t *fti_head = NULL;
 extern void fatal PROTO ((char *));
 
 
+void
+compute_directories()
+{
+    char *prefix_relative_bin_dir = "/bin";
+    char *prefix_relative_term_dir = "/lib/git/term";
+
+    bindir = xmalloc(strlen(PREFIX) + strlen(prefix_relative_bin_dir) + 1);
+    sprintf(bindir, "%s%s", PREFIX, prefix_relative_bin_dir);
+    termdir = xmalloc(strlen(PREFIX) + strlen(prefix_relative_term_dir) + 1);
+    sprintf(termdir, "%s%s", PREFIX, prefix_relative_term_dir);
+}
+
+
+/*
+ * Add $(bindir) to the PATH environment variable so that git will be
+ * able to find its auxiliary programs/scripts even when that
+ * directory is not in the PATH.  I don't think it is worth searching
+ * $(bindir) in the PATH first...
+ */
+
+void
+update_path()
+{
+    char *path = getenv("PATH");
+
+    if (path == NULL)
+	xsetenv("PATH", bindir);
+    else
+    {
+	char *new_value = xmalloc(strlen(path) + 1 + strlen(bindir) + 1);
+
+	sprintf(new_value, "%s:%s", path, bindir);
+	xsetenv("PATH", new_value);
+	xfree(new_value);
+    }
+}
+
+
 void
 display_exit_message(signame)
     char *signame;
@@ -89,7 +126,7 @@ display_exit_message(signame)
 
     fprintf(stderr, "%s %d %2d:%02d:%02d %s[%d]: exiting on %s signal\n",
 	    month_name[time->tm_mon], time->tm_mday, time->tm_hour,
-	    time->tm_min, time->tm_sec, program, (int)pid, signame);
+	    time->tm_min, time->tm_sec, program, (int)getpid(), signame);
 }
 
 
@@ -104,24 +141,19 @@ fatal_signal(signum)
     switch (signum)
     {
 	case SIGTERM:
-	case SIGQUIT:
-
-	    display_exit_message((signum == SIGTERM) ? "TERM" : "QUIT");
+	    display_exit_message("TERM");
 	    break;
 
 	case SIGHUP:
 	case SIGINT:
-
 	    display_exit_message((signum == SIGHUP) ? "HUP" : "INT");
 	    break;
 
 	case SIGSEGV:
-
 	    display_exit_message("SEGV");
 	    goto ask_report;
 
 	default:
-
 	    fprintf(stderr,
 		    "%s: got a stupid signal (%d). Unless it was a joke ...\n",
 		    program, signum);
@@ -177,9 +209,10 @@ void
 common_configuration_init()
 {
     /* Load the global .gitrc.common file.  */
-    char *configfile = xmalloc(strlen(term) + 1 + strlen(CONFIGFILE_PREFIX) +
+    char *configfile = xmalloc(strlen(termdir) + 1 +
+			       strlen(CONFIGFILE_PREFIX) +
 			       sizeof("common") + 1);
-    strcpy(configfile, term);
+    strcpy(configfile, termdir);
     strcat(configfile, CONFIGFILE_PREFIX);
     strcat(configfile, "common");
 
@@ -204,9 +237,10 @@ specific_configuration_init()
     if (configuration_init(configfile) == 0)
     {
 	xfree(configfile);
-	configfile = xmalloc(strlen(term) + 1 + strlen(CONFIGFILE_PREFIX) +
+	configfile = xmalloc(strlen(termdir) + 1 +
+			     strlen(CONFIGFILE_PREFIX) +
 			     strlen(tty_type) + 1);
-	strcpy(configfile, term);
+	strcpy(configfile, termdir);
 	strcat(configfile, CONFIGFILE_PREFIX);
 	strcat(configfile, tty_type);
 
@@ -215,9 +249,10 @@ specific_configuration_init()
 	    configuration_warning(configfile);
 
 	    xfree(configfile);
-	    configfile = xmalloc(strlen(term) + 1 + strlen(CONFIGFILE_PREFIX) +
+	    configfile = xmalloc(strlen(termdir) + 1 +
+				 strlen(CONFIGFILE_PREFIX) +
 				 sizeof("generic") + 1);
-	    strcpy(configfile, term);
+	    strcpy(configfile, termdir);
 	    strcat(configfile, CONFIGFILE_PREFIX);
 	    strcat(configfile, "generic");
 
@@ -372,7 +407,7 @@ minimize_path(path)
     if (*opath == '/')
 	*cpath++ = *opath++;
     else
-	fatal("relative path encounter");
+	fatal("relative path encountered");
 
     while (*opath)
     {
@@ -414,19 +449,6 @@ minimize_path(path)
 }
 
 
-void
-get_tty_name()
-{
-    if ((tty_name = ttyname(1)) == NULL)
-    {
-	fprintf(stderr, "%s: can't get terminal name.\n", program);
-	exit(1);
-    }
-
-    tty_name_len = strlen(tty_name);
-}
-
-
 void
 get_login_name()
 {
@@ -446,24 +468,45 @@ get_login_name()
     }
 
     login_name     = xstrdup(pwd->pw_name);
-    login_name_len = strlen(login_name);
 }
 
 
 void
 truncate_long_name(name, dest, len)
     char *name, *dest;
-    size_t len;
+    int len;
 {
-    size_t name_len = strlen(name);
+    int name_len;
 
-    if (name_len > len)
+    switch (len)
     {
-	dest[0] = dest[1] = dest[2] = '.';
-	memcpy(dest + 3, name + name_len - len + 3, len - 3);
+	case 0:
+	    break;
+
+	case 1:
+	    dest[0] = ' ';
+	    break;
+
+	case 2:
+	    dest[0] = dest[1] = ' ';
+	    break;
+
+	case 3:
+	    dest[0] = dest[1] = dest[2] = ' ';
+	    break;
+
+	default:
+	    name_len = strlen(name);
+
+	    if (name_len > len)
+	    {
+		dest[0] = dest[1] = dest[2] = '.';
+		memcpy(dest + 3, name + name_len - len + 3, len - 3);
+	    }
+	    else
+		memcpy(dest, name, name_len);
+	    break;
     }
-    else
-	memcpy(dest, name, name_len);
 }
 
 
@@ -471,10 +514,10 @@ char *
 truncate_string(path, temppath, len)
     char *path;
     char *temppath;
-    size_t len;
+    int len;
 {
     truncate_long_name(path, temppath, len - 1);
-    temppath[min(len - 1, strlen(path))] = '\0';
+    temppath[min(len - 1, (int)strlen(path))] = '\0';
     return temppath;
 }
 
@@ -605,12 +648,67 @@ putenv(entry)
     return 0;
 }
 #else
-/* I guess we are simply out of luck.  #error ?  */
+/* I guess we are simply out of luck.  */
+#error
 #endif /* NeXT */
 #endif /* HAVE_SETENV */
 #endif /* HAVE_PUTENV */
 
 
+/*
+ * Set the environment variable `variable' to the given `value'.
+ * Remove the previous value, if any.
+ * Return 0 on success, -1 on failure.
+ */
+
+int
+xsetenv(variable, value)
+    char *variable;
+    char *value;
+{
+    int result;
+
+#ifdef HAVE_PUTENV
+    {
+	char *environment_string =
+	    xmalloc(strlen(variable) + 1 + strlen(value) + 1);
+	sprintf(environment_string, "%s=%s", variable, value);
+	result = putenv(environment_string);
+    }
+#else
+#ifdef HAVE_SETENV
+    unsetenv(variable);
+    result = setenv(variable, value, 1);
+#else
+#error
+#endif /* HAVE_SETENV */
+#endif /* HAVE_PUTENV */
+
+    if (result == -1)
+	fprintf(stderr, "%s: warning: cannot add '%s' to environment\n",
+		program, variable);
+
+    return result;
+}
+
+
+/*
+ * Make sure there are only printable characters in `string'.
+ */
+
+void
+toprintable(string, length)
+    char *string;
+    size_t length;
+{
+    size_t i;
+
+    for (i = 0; i < length; i++)
+	if (!isprint(string[i]))
+	    string[i] = '?';
+}
+
+
 /*
  * Check if it is a background command.  We are doing this by looking
  * for a '&' at the end of the command string.
@@ -706,6 +804,7 @@ get_file_type_info()
 
     /* Fixed by Marian Ciobanu <ciobi@liis.sfos.ro>, October 24, 1995.  */
     if (fti_head1)
+    {
 	if (fti_head)
 	{
 	    previous->next = fti_head;
@@ -713,4 +812,5 @@ get_file_type_info()
 	}
 	else
 	    fti_head = fti_head1;
+    }
 }
