@@ -1,6 +1,6 @@
 /* panel.h -- The data structures and function prototypes used in panel.c.  */
 
-/* Copyright (C) 1993, 1994, 1995, 1996 Free Software Foundation, Inc.
+/* Copyright (C) 1993-1998 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Written by Tudor Hulubei and Andrei Pitis.  */
-
+/* $Id: panel.h,v 1.3 1998/03/10 05:12:12 tudor Exp $ */
 
 #ifndef _GIT_PANEL_H
 #define _GIT_PANEL_H
@@ -57,65 +57,63 @@
 
 
 /* Panel actions.  */
-#define act_NOACTION		0
-#define act_ENTER		1
-#define act_COPY		2
-#define act_DELETE		3
-#define act_SELECT		4
-#define act_SELECT_ALL		5
-#define act_UNSELECT_ALL	6
-#define act_TOGGLE		7
+#define act_NOACTION			 0
+#define act_ENTER			 1
+#define act_COPY			 2
+#define act_DELETE			 3
+#define act_SELECT			 4
+#define act_SELECT_ALL			 5
+#define act_UNSELECT_ALL		 6
+#define act_TOGGLE			 7
 
-#define act_ENABLE_NEXT_MODE	8
-#define act_ENABLE_OWNER_GROUP	9
-#define act_ENABLE_DATE_TIME	10
-#define act_ENABLE_SIZE		11
-#define act_ENABLE_MODE		12
-#define act_ENABLE_FULL_NAME	13
-#define act_ENABLE_ALL		14
+#define act_ENABLE_NEXT_MODE		 8
+#define act_ENABLE_OWNER_GROUP		 9
+#define act_ENABLE_DATE_TIME		10
+#define act_ENABLE_SIZE			11
+#define act_ENABLE_MODE			12
+#define act_ENABLE_FULL_NAME		13
+#define act_ENABLE_ALL			14
 
-#define act_SORT_NEXT_METHOD	15
-#define act_SORT_BY_NAME	16
-#define act_SORT_BY_EXTENSION	17
-#define act_SORT_BY_SIZE	18
-#define act_SORT_BY_DATE	19
-#define act_SORT_BY_MODE	20
-#define act_SORT_BY_OWNER_ID	21
-#define act_SORT_BY_GROUP_ID	22
-#define act_SORT_BY_OWNER_NAME	23
-#define act_SORT_BY_GROUP_NAME	24
+#define act_SORT_NEXT_METHOD		15
+#define act_SORT_BY_NAME		16
+#define act_SORT_BY_EXTENSION		17
+#define act_SORT_BY_SIZE		18
+#define act_SORT_BY_DATE		19
+#define act_SORT_BY_MODE		20
+#define act_SORT_BY_OWNER_ID		21
+#define act_SORT_BY_GROUP_ID		22
+#define act_SORT_BY_OWNER_NAME		23
+#define act_SORT_BY_GROUP_NAME		24
 
-#define act_MKDIR		25
-#define act_MOVE		26
-#define act_UP			27
-#define act_DOWN		28
-#define act_PGUP		29
-#define act_PGDOWN		30
-#define act_HOME		31
-#define act_END			32
-#define act_CHDIR		33
-#define act_REFRESH		34
-#define act_SWITCH		35
+#define act_MKDIR			25
+#define act_MOVE			26
+#define act_UP				27
+#define act_DOWN			28
+#define act_PGUP			29
+#define act_PGDOWN			30
+#define act_HOME			31
+#define act_END				32
+#define act_CHDIR			33
+#define act_REGET			34
+#define act_SWITCH			35
 
-#define act_PATTERN_SELECT	36
-#define act_PATTERN_UNSELECT	37
+#define act_PATTERN_SELECT		36
+#define act_PATTERN_UNSELECT		37
 
-#define act_SET_SCROLL_STEP	38
+#define act_SET_SCROLL_STEP		38
 
-#define act_ISEARCH_BEGIN	39
-#define act_ISEARCH_BACKWARD	40
-#define act_ISEARCH_FORWARD	41
-#define act_ISEARCH_END		42
+#define act_ISEARCH_BEGIN		39
+#define act_ISEARCH_BACKWARD		40
+#define act_ISEARCH_FORWARD		41
+#define act_ISEARCH_END			42
 
-#define act_CMPDIR		43
-
-#define act_CASE		44
-
-#define act_UP_ONE_DIR		45
-
-#define act_COMPARE		46
-
-#define act_BIN_PACKING		47
+#define act_CMPDIR			43
+#define act_CASE			44
+#define act_UP_ONE_DIR			45
+#define act_COMPARE			46
+#define act_BIN_PACKING			47
+#define act_HORIZONTAL_SCROLL_LEFT	48
+#define act_HORIZONTAL_SCROLL_RIGHT	49
 
 
 /* File sort methods.  */
@@ -210,35 +208,36 @@ typedef struct
 				   on a MSDOG direrctory.  Argh...  */
 #endif  /* HAVE_LINUX */
     int on_screen;		/* # of files on the screen.  */
-    char *path;			/* the current path.  */
-    char *temp;			/* a temporary string.  */
-    int current_entry;		/* the current file entry #.  */
-    int first_on_screen;	/* the # of the entry displayed in the
+    char *path;			/* The current path.  */
+    char *temp;			/* A temporary string.  */
+    int current_entry;		/* The current file entry #.  */
+    int first_on_screen;	/* The # of the entry displayed in the
 				   panel's first line.  */
-    size_t maxname;		/* don't bother :-) */
-    dir_entry_t *dir_entry;	/* a record for each file in the
+    int maxname;		/* Don't bother :-) */
+    dir_entry_t *dir_entry;	/* A record for each file in the
 				   current directory.  */
-    size_t isearch_length;	/* the current length of the string
+    size_t isearch_length;	/* The current length of the string
 				   used in incremental search.  */
-    xstack_t *isearch_stack;	/* the isearch stack.  */
-    int last_index;		/* used in the iterator.  */
-    int multiple_files;		/* a flag used in the iterator.  */
-    int lines, columns;		/* the panel is that big.  */
-    int x, y;			/* the panel's position.  */
-    int entries;		/* the # of files/directories.  */
-    char focus;			/* the panel's focus flag.  */
-    char visible;		/* the panel's visible flag.  */
-    char wrapped_isearch;	/* the wrapped isearch_flag.  */
-    int selected_entries;	/* the # of selected entries.  */
-    size_t pathlen;		/* the path length.  */
-    int display_mode;		/* the current display mode.  */
-    int sort_method;		/* the current sort method.  */
-    int scroll_step;		/* the scroll step.  */
-    int chkdest;		/* don't bother :-).  */
+    xstack_t *isearch_stack;	/* The isearch stack.  */
+    int last_index;		/* Used in the iterator.  */
+    int multiple_files;		/* A flag used in the iterator.  */
+    int x, y;			/* The panel's position.  */
+    int lines, columns;		/* The panel is that big.  */
+    int entries;		/* The # of files/directories.  */
+    char focus;			/* The panel's focus flag.  */
+    char visible;		/* The panel's visible flag.  */
+    char wrapped_isearch;	/* The wrapped isearch_flag.  */
+    int selected_entries;	/* The # of selected entries.  */
+    int horizontal_offset;	/* The horizontal scroll offset.  */
+    size_t pathlen;		/* The path length.  */
+    int display_mode;		/* The current display mode.  */
+    int sort_method;		/* The current sort method.  */
+    int scroll_step;		/* The scroll step.  */
+    int chkdest;		/* Don't bother :-).  */
 } panel_t;
 
 
-extern panel_t *panel_init PROTO ((int, int, int, int, char *));
+extern panel_t *panel_init PROTO ((char *));
 extern void panel_end PROTO ((panel_t *));
 extern void panel_init_iterator PROTO ((panel_t *));
 extern void panel_select_all PROTO ((panel_t *));
@@ -247,17 +246,19 @@ extern int panel_get_next PROTO ((panel_t *));
 extern void panel_set_focus PROTO ((panel_t *, int));
 extern void panel_no_optimizations PROTO ((panel_t *));
 extern int panel_action PROTO ((panel_t *, int, panel_t *, void *, int));
+extern window_t *panel_window PROTO ((panel_t *));
 extern char *panel_get_path PROTO ((panel_t *));
 extern char *panel_get_current_file_name PROTO ((panel_t *));
 extern uid_t panel_get_current_file_uid PROTO ((panel_t *));
 extern gid_t panel_get_current_file_gid PROTO ((panel_t *));
 extern mode_t panel_get_current_file_mode PROTO ((panel_t *));
 extern int panel_get_current_file_type	PROTO ((panel_t *));
-extern void panel_resize PROTO ((panel_t *, int, int, int, int));
+extern void panel_resize PROTO ((panel_t *, size_t, size_t, size_t, size_t));
 extern void panel_activate PROTO ((panel_t *));
 extern void panel_deactivate PROTO ((panel_t *));
 extern void panel_update PROTO ((panel_t *));
 extern void panel_set_wrapped_isearch_flag PROTO ((panel_t *));
+extern void panel_center_current_entry PROTO ((panel_t *));
 
 
 #endif  /* _GIT_PANEL_H */
