@@ -1,6 +1,6 @@
 /* signal.h -- Function prototypes for those stupid functions in signal.c.  */
 
-/* Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
+/* Copyright (C) 1993-1998 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,30 +17,28 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Written by Tudor Hulubei and Andrei Pitis.  */
-
+/* $Id: signals.h,v 1.5 1998/03/14 08:18:35 tudor Exp $ */
 
 #ifndef _GIT_SIGNAL_H
 #define _GIT_SIGNAL_H
 
 
+#include <signal.h>
+
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#endif
+
 #include "stdc.h"
 
 
-#define SIG_OFF         0
-#define SIG_ON          1
+extern int user_heart_attack;
 
 
-extern int UserHeartAttack;
-
-
-extern RETSIGTYPE suspend PROTO ((int));
-extern RETSIGTYPE resume PROTO ((int));
-extern RETSIGTYPE user_panic PROTO ((int));
-
-extern void signals_dfl	PROTO (());
+extern void signals_init PROTO (());
+extern void signal_handlers PROTO ((int));
 extern void signals PROTO ((int));
-extern void ignore_signals PROTO (());
-extern void restore_signals PROTO (());
+extern void service_pending_signals PROTO (());
 
 
 #endif  /* _GIT_SIGNAL_H */
